""" Use with Python 2.7
img2font.py processes a fontmap image and outputs a binary containing
the fontmap, for use with TI-84 Plus CSE vertical write modes.

Usage: img2font <infile.ext> <outfile>.bin

Note: The input fontmap image must have a white or black pixel in the upper
      left corner of the 0x00 location for calibration purposes.
      File output indicates that black pixel returns bit '1' and white bit '0'
"""
import sys
import getopt
import os
import time
from PIL import Image

#-----------------------------------------------------------------------------
def readFile(file):
        a = []
        f = open(file,'rb')
        b = f.read(1)
        while b!=b'':
                a.append(ord(b))
                b = f.read(1)
        f.close()
        return a
 
def writeFile(file,a):
        f = open(file,'wb+')
        f.write(bytearray(a))
        f.close()
        
def appendFile(file,a):
        f = open(file,'ab')
        f.write(bytearray(a))
        f.close()
#-----------------------------------------------------------------------------

if len(sys.argv)<3:
    print("Error: Not enough arguments supplied.")
    sys.exit(2)
    
infname = sys.argv[1]
outfname = sys.argv[2]
if os.path.isfile(infname)==False:
    print("Error: Input file does not exist: " + infname)
    sys.exit(2)
infile = Image.open(infname)

if infile.mode != "RGB":
    inarray = list(infile.convert("RGB").getdata())
else:
    inarray = list(infile.getdata())
w = infile.size[0]
h = infile.size[1]
s = w*h
#quickref: getpixel: infile[(y*w)+x]
#          getcoord: x=a%w ; y=a//w

print "Checking image..."

a = -1
#scan image for first instance of white or dark pixel
for x,cptr in zip(inarray,range(len(inarray))):
  if x in ((0,0,0),(255,255,255)):
    a = cptr
    break
if a==-1:
  print("Error: Input image does not contain valid pixels.")
  sys.exit(2)
#extract x,y coords from position.
cx = a%w
cy = a//w

mw = 8 #Maximum width/height to be hardcoded for now. Later on, process
mh = 8 #image file or cmd line input to change this to maybe 16x16

hbits = (((mh//8)-1)+1)*8  #ensures that number of horiz bits are always multiple of 8



#individual character arrays are (mw+1) by (mh+1).
#scan pattern: left to right 0xn0 to 0xnF, top to bottom 0x0n to 0x7n
#pixel scan:   top to bottom up to 8 for same byte left-aligned 0bNNNN----
#              left to right for a max width of 8

print "Scanning image...\n"

font_table_array = []

#number of hbits-lists in bmparray is dependent on mh.
# bmparray is [width, hbits-lists...]

for aptr in range(0x80):
  stx = cx+((aptr%0x10)*(mw+1)) 
  sty = cy+((aptr//0x10)*(mh+1))
  bmparray = [0] * (1+mh) # first entry is width of actual char
  widthactual = 0
  for x in range(mw):
    bitarray = [-1] * hbits
    for y in range(mh):
      #getpix : infile[(y*w)+x]
      px = inarray[((sty+y)*w)+(stx+x)]
      if px==(0,0,0):  #black
        bitarray[y] = 1
      if px==(255,255,255): #white
        bitarray[y] = 0
    pass
    if bitarray[0]!=-1:
      widthactual = widthactual + 1
    bmparray[x+1] = bitarray
  pass
  if widthactual==0:
    widthactual=1  #width must always be a positive integer
  bmparray[0]=widthactual
  font_table_array.append(bmparray)
pass

print "Processing image..."

outarray = []
for i in range(0x80):
  bmpa = font_table_array[i]
  for a in range(len(bmpa)):
    if isinstance(bmpa[a],list):
      tmp = 0
      lval = bmpa[a]
      for iter in range((len(lval)//8)+0):  #iterates for number of bytes in list
        tempval = 0
        for subiter in range(8):
          if lval[(iter*8)+subiter]==1:
            tbit=1
          else:
            tbit=0
          tempval = tempval + (tbit<<(8-(subiter+1)))
        outarray.append(tempval)
      pass
    else:
      outarray.append(bmpa[a])
    pass
  pass
pass

print "Writing file..."

writeFile(outfname,outarray)

print "File written to."

